"""
AI-Powered Search Example

This example demonstrates the AI-enhanced search with reranking.
Requires DASHSCOPE_API_KEY environment variable.
"""

import asyncio
import json
import sys
import os
from time import time

# Add parent directory to path for imports
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..', 'searcher', 'src'))

from useAI2Search.SearchAgent import filterAnswer


async def main():
    """Run AI-powered search example."""
    print("=== AI-Powered Search Example ===\n")
    
    # Check for API key
    if not os.getenv("DASHSCOPE_API_KEY"):
        print("ERROR: DASHSCOPE_API_KEY environment variable not set!")
        print("Please set it with: export DASHSCOPE_API_KEY='your-key'")
        return
    
    query = "AI发展趋势 2025"
    max_results = 12
    
    print(f"Searching for: {query}")
    print(f"Max results: {max_results}")
    print("Note: This may take 20-60 seconds due to AI processing...\n")
    
    # Measure time
    start_time = time()
    
    # Get AI-enhanced results
    results = await filterAnswer(query, max_results, "zh")
    
    end_time = time()
    elapsed = end_time - start_time
    
    # Display results
    print(f"\n✓ Completed in {elapsed:.2f} seconds")
    print(f"Found {len(results)} ranked results:\n")
    
    for result in results:
        print(f"Rank {result['rank']}: {result['title']}")
        print(f"   URL: {result['url']}")
        
        # Show content preview
        content = result.get('Content', '')
        if content:
            preview = content[:150].replace('\n', ' ')
            print(f"   Content: {preview}...")
        print()


if __name__ == "__main__":
    asyncio.run(main())



from agno.agent import Agent
from agno.models.dashscope import DashScope
from agno.agent import Agent
from AIWebSearcher.WebSearch.baiduSearchTool import BaiduSearchTools
import json
from pydantic import BaseModel


class SearchResult(BaseModel):
    rank: str
    title: str
    url: str

class SearchResultList(BaseModel):
    results: list[SearchResult]

class AllResult(BaseModel):
    title: str
    url: str
    mostRelativeContent: str



manager = Agent(
    model=DashScope(id="qwen-plus"),
    description="Rerank the search results based on their relevance to the topic of the user input query.",
    instructions=[
        "You are a Summizer specialized in summarizing results based on their relevance to the topic of the user input query.",
        "Ensure that your ranking is clear and well-justified, explaining why certain results are ranked higher than others.",
        "Output as json format.",
    ],
    output_schema=AllResult,
    markdown=True
)

finder1 = Agent(
    model=DashScope(id="qwen-flash"),
    description="Rerank the search results based on their relevance to the topic of the user input query.",
    instructions=[
            "遍历提供的每条结果，结合标题、摘要和 url 判断与查询主题的相关度。所有输入的文章都要保留下来",
            "输出一个 JSON 对象，键名为 results，值为按相关度排序的数组。数组元素必须包含 rank、title、url 三个字段。",
            "rank 必须从 1 开始按相关度顺序递增，不得跳号或重复。",
            "示例输出: {\"results\": [{\"rank\": \"1\", \"title\": \"...\", \"url\": \"https://...\"}]}。",
    ],
    output_schema=SearchResultList,
    markdown=False,
)
finder2 = Agent(
    model=DashScope(id="qwen-flash"),
    description="Rerank the search results based on their relevance to the topic of the user input query.",
    instructions=[
            "遍历提供的每条结果，结合标题、摘要和 url 判断与查询主题的相关度。所有输入的文章都要保留下来",
            "输出一个 JSON 对象，键名为 results，值为按相关度排序的数组。数组元素必须包含 rank、title、url 三个字段。",
            "rank 必须从 1 开始按相关度顺序递增，不得跳号或重复。",
            "示例输出: {\"results\": [{\"rank\": \"1\", \"title\": \"...\", \"url\": \"https://...\"}]}。",
    ],
    output_schema=SearchResultList,
    markdown=False,
)
finder3 = Agent(
    model=DashScope(id="qwen-flash"),
    description="Rerank the search results based on their relevance to the topic of the user input query.",
    instructions=[
            "遍历提供的每条结果，结合标题、摘要和 url 判断与查询主题的相关度。所有输入的文章都要保留下来",
            "输出一个 JSON 对象，键名为 results，值为按相关度排序的数组。数组元素必须包含 rank、title、url 三个字段。",
            "rank 必须从 1 开始按相关度顺序递增，不得跳号或重复。",
            "示例输出: {\"results\": [{\"rank\": \"1\", \"title\": \"...\", \"url\": \"https://...\"}]}。",
    ],
    output_schema=SearchResultList,
    markdown=False,
)



async def filterAnswer(query, max_results, language="zh"):
    Tools = BaiduSearchTools()
    AllResult = json.loads(Tools.baidu_search(query, max_results*2, language))
    Result1 = AllResult[0:max_results//3]
    Result2 = AllResult[max_results//3:2*max_results//3]
    Result3 = AllResult[2*max_results//3:max_results]

    filter1 = await finder1.arun(
        f"User input query: {query}. {json.dumps(Result1, indent=2, ensure_ascii=False)}",
        stream=False
    )
    filter2 = await finder2.arun(
        f"User input query: {query}. {json.dumps(Result2, indent=2, ensure_ascii=False)}",
        stream=False
    )
    filter3 = await finder3.arun(
        f"User input query: {query}. {json.dumps(Result3, indent=2, ensure_ascii=False)}",
        stream=False
    )


    finalResult = []
    if filter1.content and isinstance(filter1.content.results, list):
        finalResult.extend(item.model_dump() for item in filter1.content.results)
    if filter2.content and isinstance(filter2.content.results, list):
        finalResult.extend(item.model_dump() for item in filter2.content.results)
    if filter3.content and isinstance(filter3.content.results, list):
        finalResult.extend(item.model_dump() for item in filter3.content.results)


    finalResult = [item for item in finalResult if str(item.get('rank')) not in ('3', '4')]
    for idx, item in enumerate(finalResult, start=1):
        item['rank'] = str(idx)
        item['Content'] = await Tools.async_fetch_page(item['url'])

    return finalResult

# if __name__ == "__main__":
#     from time import time
#     import asyncio
#     beginTime = time()
#     searchRank = asyncio.run(filterAnswer("人工智能", 6))
#     endTime = time()
#     print(f"程序执行耗时: {endTime - beginTime:.2f} 秒")




import requests
from html.parser import HTMLParser
from urllib.parse import urljoin
from typing import Optional

try:
    import trafilatura
except ImportError as e:
    raise ImportError(
        "This function requires 'trafilatura'. Install with: pip install trafilatura requests"
    ) from e


def extract_text_from_url(
    url: str,
    *,
    follow_pagination: bool = True,
    pagination_limit: int = 3,
    timeout: float = 10.0,
    user_agent: Optional[str] = None,
) -> str:
    """
    提取指定网址的整页可读文本（可选跟随 rel=\"next\" 分页），实现方式与项目一致：
    - 使用 requests 抓取 HTML
    - 使用 trafilatura.extract 提取纯文本，失败时回退到原始 HTML 文本

    参数:
      - url: 目标网页
      - follow_pagination: 是否跟随 rel=\"next\" 的分页链接
      - pagination_limit: 最多跟随的分页深度（至少 1）
      - timeout: 每次 HTTP 请求超时时间（秒）
      - user_agent: 自定义 UA；不提供则使用常见浏览器 UA

    返回:
      - 拼接后的纯文本，分页之间以空行分隔；无可用文本时返回空字符串
    """

    class _RelLinkParser(HTMLParser):
        def __init__(self):
            super().__init__()
            self.href: Optional[str] = None

        def handle_starttag(self, tag, attrs):
            if self.href:
                return
            attr = {k.lower(): v for k, v in attrs}
            rel_attr = (attr.get("rel") or "").lower().split()
            if "next" in rel_attr:
                self.href = attr.get("href")

    def _find_rel_next(html: str) -> Optional[str]:
        try:
            parser = _RelLinkParser()
            parser.feed(html)
            return parser.href
        except Exception:
            return None

    headers = {
        "User-Agent": user_agent
        or (
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
            "AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/123.0.0.0 Safari/537.36"
        )
    }

    texts: list[str] = []
    visited = set()
    current = url

    for _ in range(max(1, int(pagination_limit))):
        if current in visited:
            break
        visited.add(current)

        resp = requests.get(current, headers=headers, timeout=timeout)
        resp.raise_for_status()
        raw_html = resp.text

        extracted = trafilatura.extract(
            raw_html,
            include_comments=False,
            include_tables=False,
        )
        clean_text = extracted.strip() if extracted else raw_html.strip()
        if clean_text:
            texts.append(clean_text)

        if not follow_pagination:
            break

        next_href = _find_rel_next(raw_html)
        if not next_href:
            break
        current = urljoin(current, next_href)

    return ("\n\n".join(texts)).strip()

def filter_extracted_text(   
    url: str,
    *,
    follow_pagination: bool = True,
    pagination_limit: int = 3,
    timeout: float = 10.0,
    user_agent: Optional[str] = None,
    regular_expressions: Optional[list[str]] = None,
) -> str:
    """
    提取并过滤指定网址的整页可读文本（可选跟随 rel=\"next\" 分页），实现方式与项目一致：
    - 使用 requests 抓取 HTML
    - 使用 trafilatura.extract 提取纯文本，失败时回退到原始 HTML 文本
    - 使用正则表达式过滤文本

    参数:
      - url: 目标网页
      - follow_pagination: 是否跟随 rel=\"next\" 的分页链接
      - pagination_limit: 最多跟随的分页深度（至少 1）
      - timeout: 每次 HTTP 请求超时时间（秒）
      - user_agent: 自定义 UA；不提供则使用常见浏览器 UA
      - regular_expressions: 用于过滤文本的正则表达式列表；如果为 None 或空列表，则不进行过滤
      """
    import re

    text = extract_text_from_url(
        url,
        follow_pagination=follow_pagination,
        pagination_limit=pagination_limit,
        timeout=timeout,
        user_agent=user_agent,
    )

    if not regular_expressions:
        return text

    patterns = [re.compile(pattern) for pattern in regular_expressions]
    filtered_lines = [
        line for line in text.splitlines()
        if any(pattern.search(line) for pattern in patterns)
    ]

    return "\n".join(filtered_lines).strip()




"""
Basic Search Example

This example demonstrates how to use the basic Baidu search functionality.
"""

import json
import sys
import os

# Add parent directory to path for imports
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..', 'searcher', 'src'))

from WebSearch.baiduSearchTool import BaiduSearchTools


def main():
    """Run basic search example."""
    print("=== Basic Baidu Search Example ===\n")
    
    # Initialize the search tool
    tool = BaiduSearchTools(debug=True)
    
    # Perform a search
    query = "人工智能发展现状"
    max_results = 5
    
    print(f"Searching for: {query}")
    print(f"Max results: {max_results}\n")
    
    # Get results
    results_json = tool.baidu_search(query, max_results=max_results, language="zh")
    results = json.loads(results_json)
    
    # Display results
    print(f"Found {len(results)} results:\n")
    
    for idx, result in enumerate(results, 1):
        print(f"{idx}. {result['title']}")
        print(f"   URL: {result['url']}")
        print(f"   Abstract: {result['abstract'][:100]}...")
        print()


if __name__ == "__main__":
    main()



"""
Web Content Extraction Example

This example demonstrates extracting clean text from web pages.
"""

import sys
import os

# Add parent directory to path for imports
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..', 'searcher', 'src'))

from FetchPage.fetchWeb import filter_extracted_text


def main():
    """Run content extraction example."""
    print("=== Web Content Extraction Example ===\n")
    
    # Example URL
    url = "https://www.example.com"
    
    print(f"Extracting content from: {url}\n")
    
    # Extract text
    try:
        content = filter_extracted_text(
            url,
            follow_pagination=False,
            timeout=10.0
        )
        
        print("Extracted content:")
        print("=" * 80)
        print(content[:500])  # Show first 500 characters
        print("...")
        print("=" * 80)
        print(f"\nTotal length: {len(content)} characters")
        
    except Exception as e:
        print(f"Error extracting content: {e}")


if __name__ == "__main__":
    main()



import json
from typing import Any, Dict, List, Optional
from bs4 import BeautifulSoup
import trafilatura

from agno.tools import Toolkit
from agno.utils.log import log_debug

try:
    from baidusearch.baidusearch import search  # type: ignore
except ImportError:
    raise ImportError("`baidusearch` not installed. Please install using `pip install baidusearch`")

try:
    from pycountry import pycountry
except ImportError:
    raise ImportError("`pycountry` not installed. Please install using `pip install pycountry`")


class BaiduSearchTools(Toolkit):
    """
    BaiduSearch is a toolkit for searching Baidu easily.

    Args:
        fixed_max_results (Optional[int]): A fixed number of maximum results.
        fixed_language (Optional[str]): A fixed language for the search results.
        headers (Optional[Any]): Headers to be used in the search request.
        proxy (Optional[str]): Proxy to be used in the search request.
        debug (Optional[bool]): Enable debug output.
    """

    def __init__(
        self,
        fixed_max_results: Optional[int] = None,
        fixed_language: Optional[str] = None,
        headers: Optional[Any] = None,
        proxy: Optional[str] = None,
        timeout: Optional[int] = 10,
        debug: Optional[bool] = False,
        enable_baidu_search: bool = True,
        all: bool = False,
        **kwargs,
    ):
        self.fixed_max_results = fixed_max_results
        self.fixed_language = fixed_language
        self.headers = headers
        self.proxy = proxy
        self.timeout = timeout
        self.debug = debug

        tools = []
        if all or enable_baidu_search:
            tools.append(self.baidu_search)

        super().__init__(name="baidusearch", tools=tools, **kwargs)

    def baidu_search(self, query: str, max_results: int = 5, language: str = "zh") -> str:
        """Execute Baidu search and return results

        Args:
            query (str): Search keyword
            max_results (int, optional): Maximum number of results to return, default 5
            language (str, optional): Search language, default Chinese

        Returns:
            str: A JSON formatted string containing the search results.
        """
        max_results = self.fixed_max_results or max_results
        language = self.fixed_language or language

        if len(language) != 2:
            try:
                language = pycountry.languages.lookup(language).alpha_2
            except LookupError:
                language = "zh"

        log_debug(f"Searching Baidu [{language}] for: {query}")

        results = search(keyword=query, num_results=max_results)

        res: List[Dict[str, str]] = []
        for idx, item in enumerate(results, 1):
            res.append(
                {
                    "title": item.get("title", ""),
                    "url": item.get("url", ""),
                    "abstract": item.get("abstract", "").replace("\n", " "),
                    # "rank": str(idx),
                }
            )
        return json.dumps(res, indent=2, ensure_ascii=False)

    def fetch_page(self, url: str, timeout_s: int | None = 10) -> str:
        """Fetch a single web page and return main text content.

        Uses trafilatura for better content extraction, falls back to BeautifulSoup.
        """
        try:
            import requests
            
            headers = {
                "User-Agent": (
                    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
                    "AppleWebKit/537.36 (KHTML, like Gecko) "
                    "Chrome/122.0.0.0 Safari/537.36"
                )
            }
            resp = requests.get(url, headers=headers, timeout=timeout_s)
            resp.raise_for_status()

            text = trafilatura.extract(resp.content, include_comments=False, include_tables=False)
            if text and len(text.strip()) > 100:  
                return text.strip()

            raw_text = resp.text.strip()
            return raw_text[:5000] if len(raw_text) > 5000 else raw_text

            soup = BeautifulSoup(resp.content, "html.parser")
            
            # Remove script, style, and other non-content tags
            for element in soup(["script", "style", "nav", "header", "footer", "aside", "iframe", "noscript"]):
                element.decompose()
            
            # Try to find main content area
            main_content = soup.find("main") or soup.find("article") or soup.find("div", class_=["content", "main", "article"])
            if main_content:
                text = main_content.get_text(" ", strip=True)
            else:
                text = soup.get_text(" ", strip=True)
            
            # Clean up excessive whitespace
            import re
            text = re.sub(r'\s+', ' ', text)
            
            # Limit to reasonable length (first 5000 chars)
            return text[:5000] if len(text) > 5000 else text
            
        except Exception as e:
            log_debug(f"Error fetching page {url}: {e}")
            return ""

    async def async_fetch_page(self, url: str, timeout_s: int | None = 10) -> str:
        """Fetch a single web page and return main text content.

        Uses trafilatura for better content extraction, falls back to BeautifulSoup.
        """
        try:
            import requests
            
            headers = {
                "User-Agent": (
                    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
                    "AppleWebKit/537.36 (KHTML, like Gecko) "
                    "Chrome/122.0.0.0 Safari/537.36"
                )
            }
            resp = requests.get(url, headers=headers, timeout=timeout_s)
            resp.raise_for_status()

            text = trafilatura.extract(resp.content, include_comments=False, include_tables=False)
            if text and len(text.strip()) > 100:  # Only return if we got substantial content
                return text.strip()

            raw_text = resp.text.strip()
            return raw_text[:5000] if len(raw_text) > 5000 else raw_text

            soup = BeautifulSoup(resp.content, "html.parser")
            
            # Remove script, style, and other non-content tags
            for element in soup(["script", "style", "nav", "header", "footer", "aside", "iframe", "noscript"]):
                element.decompose()
            
            # Try to find main content area
            main_content = soup.find("main") or soup.find("article") or soup.find("div", class_=["content", "main", "article"])
            if main_content:
                text = main_content.get_text(" ", strip=True)
            else:
                text = soup.get_text(" ", strip=True)
            
            # Clean up excessive whitespace
            import re
            text = re.sub(r'\s+', ' ', text)
            
            # Limit to reasonable length (first 5000 chars)
            return text[:5000] if len(text) > 5000 else text
            
        except Exception as e:
            log_debug(f"Error fetching page {url}: {e}")
            return ""





import json
from typing import Optional

from fastmcp import FastMCP
from AIWebSearcher.FetchPage.fetchWeb import filter_extracted_text
from AIWebSearcher.WebSearch.baiduSearchTool import BaiduSearchTools
from AIWebSearcher.useAI2Search.SearchAgent import filterAnswer


mcp = FastMCP("AIWebSearcher")

# @mcp.tool
# def add(a: int, b: int) -> int:
#     """Add two numbers"""
#     return a + b

@mcp.tool
def search_baidu(query: str, max_results: int = 5, language: str = "zh") -> str:
    """Execute Baidu search and return results,
    Due to the limited amount of content returned, it is recommended to use parameters of max_results with four or more

    Args:
        query (str): Search keyword
        max_results (int, optional): Maximum number of results to return, default 5
        language (str, optional): Search language, default Chinese

    Returns:
        str: A JSON formatted string containing the search results (Title, URL, abstract).
    """
    tool = BaiduSearchTools()
    return tool.baidu_search(query, max_results=max_results, language=language)

@mcp.tool
async def AI_search_baidu(query: str, max_results: int = 5, language: str = "zh") -> str:
    """Execute Baidu search using AI and return results in order, this func time cost almost 10s,
    The AI will automatically filter and sort the search results, it is recommended to use parameters of max_results with five or more

    Args:
        query (str): Search keyword
        max_results (int, optional): Maximum number of results to return, default 5
        language (str, optional): Search language, default Chinese

    Returns:
        str: A JSON formatted string containing the search results (rank, title, url, Content).
    """
    result = await filterAnswer(query, max_results, language)
    return json.dumps(result, ensure_ascii=False, indent=2)

@mcp.tool
def extractTextFromUrl(
    url: str,
    *,
    follow_pagination: bool = True,
    pagination_limit: int = 3,
    timeout: float = 10.0,
    user_agent: Optional[str] = None,
    regular_expressions: Optional[list[str]] = None,
) -> str:
    """
    Extract and filter readable text from a webpage with optional pagination support.
    
    Uses requests to fetch HTML, trafilatura to extract clean text (falls back to raw HTML),
    and regex patterns to filter content.

    Args:
        url: Target webpage URL
        follow_pagination: Whether to follow rel="next" pagination links
        pagination_limit: Maximum pagination depth (minimum 1)
        timeout: HTTP request timeout in seconds
        user_agent: Custom User-Agent header (defaults to common browser UA)
        regular_expressions: Regex patterns to filter text; no filtering if None or empty
    """
    return filter_extracted_text(
        url,
        follow_pagination=follow_pagination,
        pagination_limit=pagination_limit,
        timeout=timeout,
        user_agent=user_agent,
        regular_expressions=regular_expressions,
    )

@mcp.tool
def extractTextFromUrls(
    urls: list[str],
    *,
    follow_pagination: bool = True,
    pagination_limit: int = 3,
    timeout: float = 10.0,
    user_agent: Optional[str] = None,
    regular_expressions: Optional[list[str]] = None,
) -> str:
    """
    Extract and filter readable text from multiple webpages with optional pagination support.
    
    Uses requests to fetch HTML, trafilatura to extract clean text (falls back to raw HTML),
    and regex patterns to filter content. Regex filtering is recommended for multiple URLs.

    Args:
        urls: List of target webpage URLs
        follow_pagination: Whether to follow rel="next" pagination links
        pagination_limit: Maximum pagination depth (minimum 1)
        timeout: HTTP request timeout in seconds
        user_agent: Custom User-Agent header (defaults to common browser UA)
        regular_expressions: Regex patterns to filter text; no filtering if None or empty
    """
    results = []
    for url in urls:
        results.append(filter_extracted_text(
        url,
        follow_pagination=follow_pagination,
        pagination_limit=pagination_limit,
        timeout=timeout,
        user_agent=user_agent,
        regular_expressions=regular_expressions,
        ))
    return "\n\n".join(results)


def main() -> None:
    """Start the MCP server over stdio."""
    mcp.run(transport="stdio")


if __name__ == "__main__":
    main()

